package org.runetracker;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Window;
import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.sun.org.apache.bcel.internal.generic.NEW;

public class SettingsDialog extends JDialog {
	private static final long serialVersionUID = 1L;

	private JCheckBox autoUpdate;

	private JCheckBox autoUpdateTwiceDaily;

	private JTextField autoUpdateTime;

	private JTextField displayName;

	private static final DateFormat timeFormat = DateFormat.getTimeInstance(DateFormat.MEDIUM);

	static {
		timeFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
		timeFormat.setLenient(true);
	}

	public SettingsDialog() throws HeadlessException {
		init();
	}

	public SettingsDialog(Dialog owner, boolean modal) {
		super(owner, modal);
		init();
	}

	public SettingsDialog(Dialog owner, String title, boolean modal, GraphicsConfiguration gc) {
		super(owner, title, modal, gc);
		init();
	}

	public SettingsDialog(Dialog owner, String title, boolean modal) {
		super(owner, title, modal);
		init();
	}

	public SettingsDialog(Dialog owner, String title) {
		super(owner, title);
		init();
	}

	public SettingsDialog(Dialog owner) {
		super(owner);
		init();
	}

	public SettingsDialog(Frame owner, boolean modal) {
		super(owner, modal);
		init();
	}

	public SettingsDialog(Frame owner, String title, boolean modal, GraphicsConfiguration gc) {
		super(owner, title, modal, gc);
		init();
	}

	public SettingsDialog(Frame owner, String title, boolean modal) {
		super(owner, title, modal);
		init();
	}

	public SettingsDialog(Frame owner, String title) {
		super(owner, title);
		init();
	}

	public SettingsDialog(Frame owner) {
		super(owner);
		init();
	}

	public SettingsDialog(Window owner, ModalityType modalityType) {
		super(owner, modalityType);
		init();
	}

	public SettingsDialog(Window owner, String title, ModalityType modalityType, GraphicsConfiguration gc) {
		super(owner, title, modalityType, gc);
		init();
	}

	public SettingsDialog(Window owner, String title, ModalityType modalityType) {
		super(owner, title, modalityType);
		init();
	}

	public SettingsDialog(Window owner, String title) {
		super(owner, title);
		init();
	}

	public SettingsDialog(Window owner) {
		super(owner);
		init();
	}

	protected void init() {
		setLayout(new BorderLayout(5, 5));

		autoUpdate = new JCheckBox("Automatically update every day at", SettingsManager.getInstance().getSettings().isAutoUpdate());
		autoUpdateTime = new JTextField(timeFormat.format(new Date(SettingsManager.getInstance().getSettings().getAutoUpdateTime() * 1000)), 11);
		autoUpdateTwiceDaily = new JCheckBox("and 12 hours later (AM and PM)", SettingsManager.getInstance().getSettings().isAutoUpdateTwiceDaily());

		JPanel autoUpdateOuter = new JPanel(new FlowLayout(FlowLayout.LEADING, 5, 5));
		autoUpdateOuter.add(autoUpdate);
		autoUpdateOuter.add(autoUpdateTime);

		JPanel autoUpdateTwiceDailyOuter = new JPanel(new FlowLayout(FlowLayout.LEADING, 5, 5));
		autoUpdateTwiceDailyOuter.add(Box.createHorizontalStrut(17));
		autoUpdateTwiceDailyOuter.add(autoUpdateTwiceDaily);

		JLabel displayNameLabel = new JLabel("RuneScape display name:");
		displayName = new JTextField(SettingsManager.getInstance().getSettings().getDisplayName(), 12);

		JPanel displayNameOuter = new JPanel(new FlowLayout(FlowLayout.LEADING, 5, 5));
		displayNameOuter.add(displayNameLabel);
		displayNameOuter.add(displayName);

		// Options go in a grid above.
		/*-
		 * [ ] Automatically update every day at [|        ]
		 *   [ ] and 12 hours later (AM and PM)
		 * RuneScape display name: [|        ]
		 */
		JPanel optionsGrid = new JPanel(new GridLayout(3, 1, 5, 5));
		optionsGrid.add(autoUpdateOuter);
		optionsGrid.add(autoUpdateTwiceDailyOuter);
		optionsGrid.add(displayNameOuter);
		add(optionsGrid, BorderLayout.CENTER);

		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Settings s = SettingsManager.getInstance().getSettings();
				s.setAutoUpdate(autoUpdate.isSelected());
				s.setAutoUpdateTwiceDaily(autoUpdateTwiceDaily.isSelected());
				try {
					s.setAutoUpdateTime((int) (timeFormat.parse(autoUpdateTime.getText()).getTime() % (24 * 60 * 60 * 1000) / 1000));
				} catch (ParseException e1) {
					// Don't care, the automatic update time won't have been
					// set.
					// TODO Report this exception somehow?
					e1.printStackTrace();
				}
				s.setDisplayName(displayName.getText());
				TrayIconControllerManager.getInstance().getTrayIconController().setInformationMissing(displayName.getText().equals(""));
				dispose();
			}
		});
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		// Buttons go below.
		/*-
		 *          [ OK ] [ Cancel ]
		 */
		JPanel okCancelOuter = new JPanel(new FlowLayout(FlowLayout.TRAILING, 5, 5));
		okCancelOuter.add(ok);
		okCancelOuter.add(cancel);
		add(okCancelOuter, BorderLayout.PAGE_END);

		pack();
	}
}
