package org.runetracker;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * Holds an instance of <tt>Settings</tt> for a single user and loads/saves it
 * in a file.
 * 
 * @author Cynthia of irc.rscript.org
 */
public class SettingsManager {
	private Settings settings = new Settings();

	private static final SettingsManager SINGLETON = new SettingsManager();

	/**
	 * This constructor may only be used to create the singleton.
	 */
	private SettingsManager() {}

	public static SettingsManager getInstance() {
		return SINGLETON;
	}

	/**
	 * Loads a <tt>Settings</tt> object from the file ~/.runetracker-updaterrc,
	 * silently ignoring all errors.
	 */
	public void load() {
		try {
			ObjectInputStream data = new ObjectInputStream(new FileInputStream(new File(System.getProperty("user.home"), ".runetracker-updaterrc")));
			try {
				settings = (Settings) data.readObject();
			} finally {
				data.close();
			}
		} catch (Throwable t) {}
	}

	/**
	 * Saves this <tt>SettingsManager</tt>'s <tt>Settings</tt> object to the
	 * file ~/.runetracker-updaterrc, silently ignoring all errors.
	 */
	public void save() {
		try {
			ObjectOutputStream data = new ObjectOutputStream(new FileOutputStream(new File(System.getProperty("user.home"), ".runetracker-updaterrc")));
			try {
				data.writeObject(settings);
			} finally {
				data.close();
			}
		} catch (Throwable t) {}
	}

	public Settings getSettings() {
		return settings;
	}
}
