package org.runetracker;

import java.awt.AWTException;
import java.awt.Dialog;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.runetracker.actions.UpdateNow;

/**
 * Entry point of the RuneTracker Updater application.
 * 
 * @author Cynthia of irc.rscript.org
 */
public class UpdaterMain {
	/**
	 * Entry point of the RuneTracker Updater application.
	 * 
	 * @param args
	 *            Unused.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1) {
			JOptionPane.showMessageDialog(null, "Could not initialise the look and feel for this application.", "Unknown look and feel", JOptionPane.ERROR_MESSAGE);
		}

		try {
			if (SystemTray.isSupported()) {
				// Just starting; this icon will be replaced later.
				TrayIcon icon = new TrayIcon(Toolkit.getDefaultToolkit().createImage(UpdaterMain.class.getClassLoader().getResource("images/runetracker-updater-busy.png")));
				icon.setImageAutoSize(true);
				try {
					SystemTray.getSystemTray().add(icon);
				} catch (AWTException e) {
					JOptionPane.showMessageDialog(null, "This application requires a graphical user interface and support for a notification area with icons.\n\nRight now, it seems to be gone.", "System tray?", JOptionPane.ERROR_MESSAGE);
					return;
				}

				// Load settings.
				SettingsManager.getInstance().load();
				// Save settings on exit.
				Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
					@Override
					public void run() {
						SettingsManager.getInstance().save();
					}
				}));

				// Automatically update the user's data if he or she set this
				// application to do so.
				new Thread(new AutomaticUpdater()).start();

				TrayIconControllerManager.getInstance().setTrayIconController(new TrayIconController(icon));
				TrayIconControllerManager.getInstance().getTrayIconController().setInformationMissing(SettingsManager.getInstance().getSettings().getDisplayName().equals(""));

				/* If the display name is missing, show settings right away. */
				if (TrayIconControllerManager.getInstance().getTrayIconController().isInformationMissing()) {
					SettingsDialog sd = new SettingsDialog(null, "RuneTracker Updater: Settings", ModalityType.APPLICATION_MODAL);
					sd.setVisible(true);
				}

				PopupMenu menu = new PopupMenu();
				icon.addActionListener(new UpdateNow());

				MenuItem updateNowItem = new MenuItem("Update now");
				updateNowItem.addActionListener(new UpdateNow());
				menu.add(updateNowItem);

				MenuItem settingsItem = new MenuItem("Settings...");
				settingsItem.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						SettingsDialog sd = new SettingsDialog(null, "RuneTracker Updater: Settings", ModalityType.APPLICATION_MODAL);
						sd.setVisible(true);
					}
				});
				menu.add(settingsItem);

				menu.addSeparator();

				MenuItem quitItem = new MenuItem("Quit");
				quitItem.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						System.exit(0);
					}
				});
				menu.add(quitItem);

				icon.setPopupMenu(menu);
				icon.setToolTip("RuneTracker Updater");
			} else {
				JOptionPane.showMessageDialog(null, "This application requires a graphical user interface and support for a notification area with icons.\n\nExamples of these are Microsoft Windows, Linux, Mac OS and Solaris.", "Unsupported platform", JOptionPane.ERROR_MESSAGE);
				return;
			}
		} catch (final NoClassDefFoundError ncdfe) {
			JOptionPane.showMessageDialog(null, "This application requires Java SE 6, which supports notification area icons.\n\nPlease update your Java at http://www.java.com/", "Outdated Java", JOptionPane.ERROR_MESSAGE);
			return;
		}
	}
}
